<?php
/*
Plugin Name: TablePress Debug Extension
Plugin URI: https://tablepress.org/
Description: Custom Extension for diagnosing and debugging TablePress problems
Version: 1.2
Author: Tobias Bäthge
Author URI: https://tablepress.org/
Update URI: https://tablepress.org/
*/

// Prohibit direct script loading.
defined( 'ABSPATH' ) || die( 'No direct script access allowed!' );

/**
 * TablePress Debug Controller class
 * @package TablePress
 * @subpackage Views
 * @author Tobias Bäthge
 * @since 1.0
 */
class TablePress_Debug_Controller {

	/**
	 * Class constructor.
	 *
	 * @since 1.0
	 */
	public function __construct() {
		// Hook into TablePress, if we are in the Admin Controller loading process.
		if ( is_admin() && ! ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
			add_action( 'tablepress_run', array( $this, 'run' ) );
		}
	}

	/**
	 * Starts the TablePress Debug Controller, which is run when TablePress is run.
	 *
	 * @since 1.0
	 */
	public function run() {
		// Only allow access to the Debug Extension for Admins.
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		add_filter( 'tablepress_load_file_full_path', array( $this, 'change_debug_view_full_path' ), 10, 3 );
		add_filter( 'tablepress_admin_view_actions', array( $this, 'add_view_action_debug' ) );
		add_action( 'admin_post_tablepress_debug', array( $this, 'handle_post_action_debug' ) );
		add_filter( 'tablepress_view_data', array( $this, 'add_debug_view_data' ), 10, 2 );

		add_filter( 'tablepress_post_type_args', array( $this, 'extend_post_type_args' ) );

		add_action( 'load-post.php', array( $this, 'load_post_page_filters' ) );
		add_action( 'load-post-new.php', array( $this, 'load_post_page_filters' ) );

		// Turn off "WordPress should correct invalidly nested XHTML automatically" to not mess with saving.
		add_filter( 'pre_option_use_balanceTags', '__return_zero' );
		// Turn off automatic addition of `rel="noopener noreferer"` to `target="_blank"`, as it breaks the JSON code.
		if ( function_exists( 'wp_remove_targeted_link_rel_filters' ) ) {
			wp_remove_targeted_link_rel_filters();
		}
	}

	/**
	 * Adjusts the path from which the class PHP file is loaded.
	 *
	 * @since 1.0
	 *
	 * @param string $full_path Full path of the class file.
	 * @param string $file      File name of the class file.
	 * @param string $folder    Folder name of the class file.
	 * @return string Modified full path.
	 */
	public function change_debug_view_full_path( $full_path, $file, $folder ) {
		if ( 'view-debug.php' === $file ) {
			$full_path = plugin_dir_path( __FILE__ ) . $file;
		}
		return $full_path;
	}

	/**
	 * Adds the Debug view to the list of views in TablePress.
	 *
	 * @since 1.0
	 *
	 * @param array $view_actions List of views.
	 * @return array Modified list of views.
	 */
	public function add_view_action_debug( $view_actions ) {
		$view_actions['debug'] = array(
			'show_entry' => true,
			'page_title' => 'Debug',
			'admin_menu_title' => 'Debug',
			'nav_tab_title' => 'Debug',
			'required_cap' => 'manage_options', // Only grant access to the Debug area for admins.
		);
		// If user can not access an action view by default, at least grant access to admins.
		foreach ( $view_actions as $action => $action_details ) {
			if ( ! current_user_can( $action_details['required_cap'] ) ) {
				$view_actions[ $action ]['required_cap'] = 'manage_options';
			}
		}

		return $view_actions;
	}

	/**
	 * Save changes on the "Debug" screen.
	 *
	 * @since 1.0
	 */
	public function handle_post_action_debug() {
		TablePress::check_nonce( 'debug' );

		if ( empty( $_POST['debug'] ) || ! is_array( $_POST['debug'] ) ) {
			TablePress::redirect( array( 'action' => 'debug', 'message' => 'error_save' ) );
		} else {
			$debug = stripslashes_deep( $_POST['debug'] );
		}

		/**
		 * Either delete Plugin Options or set it to new value.
		 */
		if ( '' === $debug['plugin_options'] ) {
			delete_option( 'tablepress_plugin_options' );
		} else {
			update_option( 'tablepress_plugin_options', $debug['plugin_options'] );
		}

		/**
		 * Either delete User Options or set it to new value.
		 */
		if ( '' === $debug['user_options'] ) {
			delete_user_option( get_current_user_id(), 'tablepress_user_options', false );
		} else {
			update_user_option( get_current_user_id(), 'tablepress_user_options', wp_slash( $debug['user_options'] ), false );
		}

		/**
		 * Sophisticated way to change Tables option.
		 */
		$current_value = get_option( 'tablepress_tables' ); // for check below
		$new_tables = array();
		$new_tables['last_id'] = (int) $debug['last_id'];
		$new_tables['table_post'] = array();
		if ( isset( $debug['table_post'] ) && isset( $debug['table_post']['new_table_id'] ) ) {
			$count = count( $debug['table_post']['new_table_id'] );
			for( $i = 0; $i < $count; $i++ ) {
				if ( ( '' === $debug['table_post']['new_table_id'][ $i ] ) || ( '' === $debug['table_post']['new_post_id'][ $i ] ) ) {
					continue;
				}
				$new_tables['table_post'][ (string) $debug['table_post']['new_table_id'][ $i ] ] = (int) $debug['table_post']['new_post_id'][ $i ];
			}
		}
		TablePress::$model_table->_debug_update_tables( $new_tables );

		// Either delete Tables option or set it to new value, if it differs from the previous value.
		if ( '' === $debug['tables_option'] ) {
			delete_option( 'tablepress_tables' );
		} elseif ( $current_value !== $debug['tables_option'] ) {
			update_option( 'tablepress_tables', $debug['tables_option'] );
		}

		/**
		 * Maybe clear all meta box related settings.
		 */
		if ( isset( $debug['reset_meta_boxes'] ) && ( 'true' === $debug['reset_meta_boxes'] ) ) {
			foreach( array( 'list', 'add', 'import', 'export', 'options', 'about', 'edit', 'debug' ) as $action ) {
				delete_user_option( get_current_user_id(), "meta-box-order_tablepress_{$action}", true );
				delete_user_option( get_current_user_id(), "screen_layout_tablepress_{$action}", true );
				delete_user_option( get_current_user_id(), "closedpostboxes_tablepress_{$action}", true );
				delete_user_option( get_current_user_id(), "metaboxhidden_tablepress_{$action}", true );
			}
		}

		/**
		 * Maybe clear all capability related settings.
		 */
		if ( isset( $debug['reset_capabilities'] ) && ( 'true' === $debug['reset_capabilities'] ) ) {
			// Remove all caps from all roles that start with "tablepress_".
			global $wp_roles;
			foreach ( $wp_roles->get_names() as $role => $role_name ) {
				$role = get_role( $role );
				if ( empty( $role ) ) {
					continue;
				}

				foreach ( $role->capabilities as $cap => $cap_granted ) {
					if ( 0 === strpos( $cap, 'tablepress_' ) ) {
						$role->remove_cap( $cap );
					}
				}
			}
			// Re-add our standard capabilities.
			TablePress::$model_options->add_access_capabilities();
		}

		/**
		 * Maybe clear all Table Output Caches.
		 */
		if ( isset( $debug['clear_table_output_caches'] ) && ( 'true' === $debug['clear_table_output_caches'] ) ) {
			TablePress::$model_table->invalidate_table_output_caches();
		}

		TablePress::redirect( array( 'action' => 'debug', 'message' => 'success_save' ) );
	}

	/**
	 * Adds the necessary data to the view data.
	 *
	 * @since 1.0
	 *
	 * @param array  $data   View data.
	 * @param string $action The current action.
	 * @return array The extended view data, if $action is the Extension's action.
	 */
	public function add_debug_view_data( $data, $action ) {
		// Register action to show memory usage on all TablePress screens.
		add_filter( 'admin_footer_text', array( $this, 'admin_footer_memory_usage' ), 11 );

		if ( 'debug' !== $action ) {
			return $data;
		}

		$data['tables'] = TablePress::$model_table->_debug_get_tables();
		$data['counts'] = TablePress::$model_table->count_tables( false );
		$data['posts_without_table_id'] = $this->get_posts_without_table_id();
		$data['debug']['plugin_options'] = json_encode( TablePress::$model_options->_debug_get_plugin_options() );
		$data['debug']['user_options'] = json_encode( TablePress::$model_options->_debug_get_user_options() );
		$data['debug']['tables_option'] = json_encode( $data['tables'] );
		return $data;
	}

	/**
	 * Gets all post IDs for tables that are in the database, but not in the "table_post" option.
	 *
	 * @since 1.2
	 *
	 * @return array List of post IDs of tables that are not in the "table_post" option.
	 */
	public function get_posts_without_table_id() {
		$all_post_ids = get_posts( array(
			'fields'          => 'ids',
			'post_type'       => 'tablepress_table',
			'post_status'     => 'publish',
			'posts_per_page'  => -1,
			'orderby'         => 'ID',
			'order'           => 'ASC',
		) );

		$tables = TablePress::$model_table->_debug_get_tables();
		$post_ids = array_values( $tables['table_post'] );

		return array_diff( $all_post_ids, $post_ids );
	}

	/**
	 * Extends the arguments for the TablePress Custom Post Type, so that we get a UI in the admin area.
	 *
	 * @since 1.0
	 *
	 * @param array $post_type_args Post type arguments.
	 * @return array Modified post type arguments.
	 */
	public function extend_post_type_args( $post_type_args ) {
		$post_type_args['show_ui'] = true;
		$post_type_args['show_in_admin_bar'] = false;
		$post_type_args['labels']['name'] = 'TP Tables';
		$post_type_args['supports'][] = 'custom-fields';
		unset( $post_type_args['capability_type'] ); // Turn off our capability checking for WP's regular CPT UI.
		unset( $post_type_args['map_meta_cap'] ); // necessary to get regular mapping of "edit_post" and other capabilities.
		return $post_type_args;
	}

	/**
	 * Modifies what's available on the "Edit" screen for the raw table CPT.
	 * Disable the visual editor and make Post Meta fields visible.
	 *
	 * @since 1.0
	 */
	public function load_post_page_filters() {
		add_filter( 'user_can_richedit', array( $this, '_disable_visual_editor' ) );
		add_filter( 'is_protected_meta', array( $this, '_make_post_meta_visible' ), 10, 3 );
	}

	/**
	 * Disables the visual editor on the "Edit" screen.
	 *
	 * @since 1.0
	 *
	 * @param bool $default Whether the visual editor is enabled.
	 * @param bool Whether the visual editor is enabled.
	 */
	public function _disable_visual_editor( $default ) {
		global $post;
		if ( 'tablepress_table' === get_post_type( $post ) ) {
			$default = false;
		}
		return $default;
	}

	/**
	 * Makes the Post Meta fields visible on the "Edit" screen.
	 *
	 * @since 1.0
	 *
	 * @param bool   $protected Whether the post meta field is hidden.
	 * @param string $meta_key  The post meta key.
	 * @param string $meta_type The post meta type.
	 * @return bool Whether the post meta field is hidden.
	 */
	public function _make_post_meta_visible( $protected, $meta_key, $meta_type ) {
		if ( in_array( $meta_key, array( '_tablepress_table_options', '_tablepress_table_visibility', true ) ) ) {
			$protected = false;
		}
		return $protected;
	}

	/**
	 * Shows the memory usage in the footer of every TablePress admin screen.
	 *
	 * @since 1.0
	 *
	 * @param string $content Current admin footer content.
	 * @return string New admin footer content.
	 */
	public function admin_footer_memory_usage( $content ) {
		$memory_limit = (int) @ini_get( 'memory_limit' );
		$used_memory = memory_get_peak_usage( true );
		$used_memory = round( ( $used_memory / ( 1024 * 1024 ) ), 2 );
		$content .= " &bull; Memory: {$used_memory} of {$memory_limit} MB";
		return $content;
	}

} // class TablePress_Debug_Controller

// Initialize the Debug Extensions.
new TablePress_Debug_Controller();
