<?php
/**
 * Debug View
 *
 * @package TablePress
 * @subpackage Views
 * @author Tobias Bäthge
 * @since 1.0
 */

// Prohibit direct script loading.
defined( 'ABSPATH' ) || die( 'No direct script access allowed!' );

/**
 * Debug View class
 * @package TablePress
 * @subpackage Views
 * @author Tobias Bäthge
 * @since 1.0
 */
class TablePress_Debug_View extends TablePress_View {

	/**
	 * Sets up the view with data and do things that are specific for this view.
	 *
	 * @since 1.0
	 *
	 * @param string $action Action for this view.
	 * @param array $data Data for this view.
	 */
	public function setup( $action, array $data ) {
		parent::setup( $action, $data );

		$this->add_header_message( '<strong>' . __( 'This page is for experts only.', 'tablepress' ) . ' ' . __( 'Do not make any changes, unless you know exactly what you are doing!', 'tablepress' ) . '</strong>', 'notice-error not-dismissible' );

		$this->process_action_messages( array(
			'success_save' => __( 'Debug Changes saved successfully.', 'tablepress' ),
			'error_save' => __( 'Error: Debug Changes could not be saved.', 'tablepress' )
		) );

		$this->add_meta_box( 'table-integrity', __( 'Table Data Integrity', 'tablepress' ), array( $this, 'postbox_table_integrity' ), 'normal' );
		$this->add_meta_box( 'plugin-options', __( 'Plugin Options', 'tablepress' ), array( $this, 'postbox_plugin_options' ), 'normal' );
		$this->add_meta_box( 'user-options', __( 'User Options', 'tablepress' ), array( $this, 'postbox_user_options' ), 'normal' );
		$this->add_meta_box( 'tables-option', __( 'Tables Option', 'tablepress' ), array( $this, 'postbox_tables_option' ), 'normal' );
		$this->add_meta_box( 'reset-options', __( 'Reset Options', 'tablepress' ), array( $this, 'postbox_reset_options' ), 'normal' );
		$this->add_meta_box( 'table_post', __( 'Table to Post connections', 'tablepress' ), array( $this, 'postbox_table_post' ), 'normal' );
		$this->add_text_box( 'submit', array( $this, 'textbox_submit_button' ), 'submit' );
	}

	/**
	 * Displays the post box for the Table Integrity checks.
	 *
	 * @since 1.1
	 *
	 * @param array $data Data for this screen.
	 * @param array $box  Information about the meta box.
	 */
	public function postbox_table_integrity( $data, $box ) {
		if ( empty( $data['tables'] ) || ! is_array( $data['tables'] ) ) {
			_e ( 'No tables were found.', 'tablepress' );
			return;
		}

		// Load all table IDs and prime post meta cache for cached access to options and visibility settings of the tables, don't run filter hook.
		TablePress::$model_table->load_all( true, false ); // The return value is discarded, as it skips tables that could not be loaded from the database.

		$tables_with_issues = 0;

		echo __( 'Detected table integrity issues:', 'tablepress' ) . '<br /><hr />';

		foreach ( $data['tables']['table_post'] as $table_id => $post_id ) {
			// Load table, with table data, options, and visibility settings.
			$table = TablePress::$model_table->load( $table_id, true, true );

			$table_link = '<a href="' . esc_url( TablePress::url( array( 'action' => 'edit', 'table_id' => $table_id ) ) ) . '">' . $table_id . '</a>';
			$post_link = '<a href="' . esc_url( get_edit_post_link( $post_id ) ) . '">' . $post_id . '</a>';

			if ( is_wp_error( $table ) ) {
				echo "Table “{$table_link}” (post ID {$post_link}) could not be loaded.<br />";
				++$tables_with_issues;
				continue;
			}

			if ( isset( $table['is_corrupted'] ) && $table['is_corrupted'] ) {
				echo "Table “{$table_link}” (post ID {$post_link}) is corrupted.<br />";
				++$tables_with_issues;
				continue;
			}

			if ( isset( $table['is_corrupted'] ) && $table['is_corrupted'] ) {
				echo "Table “{$table_link}” (post ID {$post_link}) is corrupted.<br />";
				++$tables_with_issues;
				continue;
			}

			// String fields.
			foreach ( array( 'id', 'name', 'description', 'last_modified' ) as $field ) {
				if ( ! isset( $table[ $field ] ) || ! is_string( $table[ $field ] ) ) {
					echo "Table “{$table_link}” (post ID {$post_link}) does not have a valid <code>{$field}</code> entry.<br />";
					++$tables_with_issues;
					continue 2;
				}
			}

			// Integer/string fields.
			foreach ( array( 'author' ) as $field ) {
				if ( ! isset( $table[ $field ] ) || ! is_numeric( $table[ $field ] ) ) {
					echo "Table “{$table_link}” (post ID {$post_link}) does not have a valid <code>{$field}</code> entry.<br />";
					++$tables_with_issues;
					continue 2;
				}
			}

			// Array fields.
			foreach ( array( 'data', 'options', 'visibility' ) as $field ) {
				if ( ! isset( $table[ $field ] ) || ! is_array( $table[ $field ] ) || empty( $table[ $field ] ) ) {
					echo "Table “{$table_link}” (post ID {$post_link}) does not have a valid <code>{$field}</code> entry.<br />";
					++$tables_with_issues;
					continue 2;
				}
			}

			// Table data needs to be ok.
			$counts = array();
			$counts['rows'] = count( $table['data'] );
			if ( 0 === $counts['rows'] ) {
				echo "Table “{$table_link}” (post ID {$post_link}) does not have a valid number of rows.<br />";
				++$tables_with_issues;
				continue;
			}
			$counts['columns'] = count( $table['data'][0] );
			if ( 0 === $counts['columns'] ) {
				echo "Table “{$table_link}” (post ID {$post_link}) does not have a valid number of columns.<br />";
				++$tables_with_issues;
				continue;
			}

			// Visibility fields.
			foreach ( array( 'rows', 'columns' ) as $field ) {
				if ( ! isset( $table['visibility'][ $field ] ) || ! is_array( $table['visibility'][ $field ] ) ) {
					echo "Table “{$table_link}” (post ID {$post_link}) does not have a valid <code>{$field}</code> visibility entry.<br />";
					++$tables_with_issues;
					continue 2;
				}
				if ( $counts[ $field ] !== count( $table['visibility'][ $field ] ) ) {
					echo "Table “{$table_link}” (post ID {$post_link}) does not have a properly sized <code>{$field}</code> visibility entry.<br />";
					++$tables_with_issues;
					continue 2;
				}
			}
		}

		if ( 0 === $tables_with_issues ) {
			_e( 'No issues detected.', 'tablepress' );
		} else {
			echo "<br /><strong>There are {$tables_with_issues} tables with table integrity issues!</strong>";
		}
	}

	/**
	 * Displays the post box for the Plugin Options field.
	 *
	 * @since 1.0
	 *
	 * @param array $data Data for this screen.
	 * @param array $box  Information about the meta box.
	 */
	public function postbox_plugin_options( $data, $box ) {
		echo 'tablepress_plugin_options (JSON):<br /><textarea class="large-text" name="debug[plugin_options]">' . esc_textarea( $data['debug']['plugin_options'] ) . '</textarea>';
	}

	/**
	 * Displays the post box for the User Options field.
	 *
	 * @since 1.0
	 *
	 * @param array $data Data for this screen.
	 * @param array $box  Information about the meta box.
	 */
	public function postbox_user_options( $data, $box ) {
		echo 'tablepress_user_options (JSON):<br /><textarea class="large-text" name="debug[user_options]">' . esc_textarea( $data['debug']['user_options'] ) . '</textarea>';
		echo '<br /><br />';
		echo 'meta-box-order_tablepress_edit:<br /><textarea class="large-text" readonly rows="1">' . esc_textarea( var_export( get_user_option( 'meta-box-order_tablepress_edit' ), true ) ) . '</textarea>';
		echo '<br />';
		echo 'screen_layout_tablepress_edit:<br /><textarea class="large-text" readonly rows="1">' . esc_textarea( var_export( get_user_option( 'screen_layout_tablepress_edit' ), true ) ) . '</textarea>';
		echo '<br />';
		echo 'closedpostboxes_tablepress_edit:<br /><textarea class="large-text" readonly rows="1">' . esc_textarea( var_export( get_user_option( 'closedpostboxes_tablepress_edit' ), true ) ) . '</textarea>';
		echo '<br />';
		echo 'metaboxhidden_tablepress_edit:<br /><textarea class="large-text" readonly rows="1">' . esc_textarea( var_export( get_user_option( 'metaboxhidden_tablepress_edit' ), true ) ) . '</textarea>';
	}

	/**
	 * Displays the post box for the TablePress Tables field.
	 *
	 * @since 1.0
	 *
	 * @param array $data Data for this screen.
	 * @param array $box  Information about the meta box.
	 */
	public function postbox_tables_option( $data, $box ) {
		echo 'tablepress_tables (JSON):<br /><textarea class="large-text" name="debug[tables_option]">' . esc_textarea( $data['debug']['tables_option'] ) . '</textarea><br />Changes this directly takes precedence over using the interface below!';
	}

	/**
	 * Displays the post box for the various reset options.
	 *
	 * @since 1.0
	 *
	 * @param array $data Data for this screen.
	 * @param array $box  Information about the meta box.
	 */
	public function postbox_reset_options( $data, $box ) {
		echo '<input type="checkbox" name="debug[reset_meta_boxes]" value="true" /> Reset Meta Boxes<br />';
		echo '<input type="checkbox" name="debug[reset_capabilities]" value="true" /> Reset Capabilities<br />';
		echo '<input type="checkbox" name="debug[clear_table_output_caches]" value="true" /> Clear Table Output Caches';
	}

	/**
	 * Displays the post box for the table ID to post ID relationship.
	 *
	 * @since 1.0
	 *
	 * @param array $data Data for this screen.
	 * @param array $box  Information about the meta box.
	 */
	public function postbox_table_post( $data, $box ) {
			echo '<p>';
			_e( 'Tables', 'tablepress' );
			echo '<br />';
			printf( __( 'in list: <strong>%s</strong>', 'tablepress' ), $data['counts']['list'] );
			echo '&nbsp;&nbsp;&middot;&nbsp;&nbsp;';
			printf( __( 'in DB: <strong>%s</strong>', 'tablepress' ), $data['counts']['db'] );
			echo '</p>';

			if ( ! empty( $data['posts_without_table_id'] ) ) {
				echo '<p>';
				echo '<strong>';
				_e( 'There are tables in the database without a table ID:', 'tablepress' );
				echo '</strong>';
				echo '<br />';
				$post_links = array();
				foreach ( $data['posts_without_table_id'] as $post_id ) {
					$post_links[] = '<a href="' . esc_url( get_edit_post_link( $post_id ) ) . '">' . $post_id . '</a>';
				}
				echo implode( ', ', $post_links );
				echo '</p>';
			}

		if ( ! empty( $data['tables'] ) && is_array( $data['tables'] ) ) {
			echo '<p>';
			$last_id_field = "<input type=\"text\" name=\"debug[last_id]\" id=\"debug_last_table_id\" class=\"small-text\" value=\"{$data['tables']['last_id']}\" style=\"width: 50px;\" />";;
			printf( __( 'Last Table ID: %s', 'tablepress' ), $last_id_field );
			echo '</p>';

			echo '<details><summary style="cursor:pointer;">Table ID to post ID map</summary>';
			echo '<table id="table_post_fields" style="width:250px;">' . "\n";
			echo '<thead><th scope="col" align="left">' . __('Table ID', 'TablePress') . '</th><th scope="col" align="left">' . __('Post ID', 'TablePress') . '</th><th></th></thead>' . "\n";
			echo '<tbody>' . "\n";
			foreach ( $data['tables']['table_post'] as $table_id => $post_id ) {
				$table_id_field = "<input type=\"text\" name=\"debug[table_post][new_table_id][]\" class=\"small-text\" value=\"{$table_id}\" />";
				$post_id_field = "<input type=\"text\" name=\"debug[table_post][new_post_id][]\" class=\"small-text\" value=\"{$post_id}\" />";
				$delete_link = '<input type="button" class="button button-small delete-list-entry hide-if-no-js" value="Delete entry" />';
				echo "\t<tr><td>{$table_id_field}</td><td>{$post_id_field}</td><td>{$delete_link}</td></tr>\n";
			}
			echo '</tbody></table>' . "\n";
			echo '<br />';
			echo '<input type="button" class="button button-small add-list-entry hide-if-no-js" value="Add entry" />';
			echo '</details>';
			add_action( 'admin_footer', array( $this, '_add_debug_js' ) );
		} else {
			echo '<p>';
			_e ( 'No tables were found.', 'tablepress' );
			echo '</p>';
		}
	}

	/**
	 * Prints debug JS to the footer (added to page in admin footer by postbox_table_post()).
	 *
	 * @since 1.0
	 */
	public function _add_debug_js() {
		echo <<<JS
<script>
jQuery(document).ready( function($) {
	$( '#tablepress_debug-table_post' )
		.on( 'click', '.delete-list-entry', function() {
			$( this ).closest( 'tr' ).remove();
		} )
		.on( 'click', '.add-list-entry', function() {
			$( '#table_post_fields' ).append( '<tr><td><input type="text" name="debug[table_post][new_table_id][]" class="small-text" value="" /></td><td><input type="text" name="debug[table_post][new_post_id][]" class="small-text" value="" /></td><td><input type="button" class="button button-small delete-list-entry hide-if-no-js" value="Delete entry" /></td></tr>' );
		} );
});
</script>
JS;
	}

} // class TablePress_Debug_View
